// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_messages.dataserver;

/// Data table
class DataTable {
  /// Data is retrieved from a remote source.
  final bool remote;

  /// Data is computed rather than retrieved from a storage.
  final bool computed;

  /// Data should be cached locally by the DataServer.
  final bool localCache;

  /// Primary key
  final PrimaryKey primaryKey;

  /// Secondary table keys
  final List<TableKey> keys;

  /// Table values
  final List<TableValue> values;

  /// Parameters that are required to request data.
  final Set<String> parameters;

  /// Constructor
  const DataTable(
      {this.remote,
      this.computed,
      this.localCache,
      this.primaryKey,
      this.keys,
      this.values,
      this.parameters});

  /// Construct from JSON data.
  factory DataTable.fromJson(Map<String, dynamic> data) {
    var keys = data['keys'] as List<Map<String, dynamic>>;
    var values = data['values'] as List<Map<String, dynamic>>;
    return new DataTable(
        remote: data['remote'],
        computed: data['computed'],
        localCache: data['localCache'],
        primaryKey:
            new PrimaryKey.fromJson(data['primaryKey'] as Map<String, dynamic>),
        keys: new List<TableKey>.generate(
            keys.length, (int i) => new TableKey.fromJson(keys[i])),
        values: new List<TableValue>.generate(
            values.length, (int i) => new TableValue.fromJson(values[i])),
        parameters: (data['parameters'] as List<String>).toSet());
  }

  /// Convert to JSON compatible data.
  Map<String, dynamic> toJson() {
    return {
      'remote': remote,
      'computed': computed,
      'localCache': localCache,
      'primaryKey': primaryKey,
      'keys': keys,
      'values': values,
      'parameters': parameters
    };
  }
}
