// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

// THIS FILE IS AUTO-GENERATED, PLEASE DO NOT MODIFY BY HAND.

part of molview_messages.v0;

/// DataServer messages
/// ===================
/// The DataServer is the central, in-memory data storage in MolView. The
/// DataServer manages the retrieval of data records from external databases and
/// web-services. It can also be used to integrate local resources with external
/// resources. Modules can submit new data tables to the DataServer. The
/// DataServer takes care of interconnecting tables and finding the best way to
/// get the data that is requested by modules. To do so the DataServer stores a
/// map of keys that link to the most recent data in the application. These keys
/// can be used to retrieve related data.

/// Activate a new resource table in the DataServer
class ActivateTable extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.ActivateTable';

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Table information
  final DataTable table;

  /// Constructor
  ActivateTable(this.database, this.table, this.table) : super(id);

  /// Construct from passive data structure.
  factory ActivateTable.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    DataTable table;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is Map) {
      table =
          new DataTable.fromPds(new Map<String, dynamic>.from(pds['table']));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    return new ActivateTable(database, table, table);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'database': database, 'table': table, 'table': table.toPds()};
  }
}

/// Message handler for ActivateTable
class ActivateTableHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<ActivateTable> _callback;

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Constructor
  ActivateTableHandler(this._callback, {this.database: null, this.table: null})
      : super(ActivateTable.id);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    ActivateTable message =
        data is ActivateTable ? data : new ActivateTable.fromPds(data.toPds());

    if ((database == null || database == message.database) &&
        (table == null || table == message.table)) {
      return [_callback(message)];
    } else {
      return [];
    }
  }
}

/// A table activation was rejected
class TableRejected extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.TableRejected';

  /// Rejection reason
  final TableRejectionReason reason;

  /// User friendly rejection message
  final String message;

  /// Constructor
  TableRejected(this.reason, this.message) : super(id);

  /// Construct from passive data structure.
  factory TableRejected.fromPds(final Map<String, dynamic> pds) {
    TableRejectionReason reason;
    String message;

    // Read reason.
    if (pds.containsKey('reason') && pds['reason'] is int) {
      reason = TableRejectionReason.values[pds['reason']];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'reason'");
    }

    // Read message.
    if (pds.containsKey('message') && pds['message'] is String) {
      message = pds['message'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'message'");
    }

    return new TableRejected(reason, message);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'reason': reason.index, 'message': message};
  }
}

/// Message handler for TableRejected
class TableRejectedHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<TableRejected> _callback;

  /// Rejection reason
  final TableRejectionReason reason;

  /// Constructor
  TableRejectedHandler(this._callback, {this.reason: null})
      : super(TableRejected.id);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    TableRejected message =
        data is TableRejected ? data : new TableRejected.fromPds(data.toPds());

    if ((reason == null || reason == message.reason)) {
      return [_callback(message)];
    } else {
      return [];
    }
  }
}

/// A new table was registered
class TableRegistered extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.TableRegistered';

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Resource table details
  final DataTable table;

  /// Constructor
  TableRegistered(this.database, this.table, this.table) : super(id);

  /// Construct from passive data structure.
  factory TableRegistered.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    DataTable table;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is Map) {
      table =
          new DataTable.fromPds(new Map<String, dynamic>.from(pds['table']));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    return new TableRegistered(database, table, table);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'database': database, 'table': table, 'table': table.toPds()};
  }
}

/// Message handler for TableRegistered
class TableRegisteredHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<TableRegistered> _callback;

  /// Database name
  final String database;

  /// Table name
  final String table;

  /// Constructor
  TableRegisteredHandler(this._callback,
      {this.database: null, this.table: null})
      : super(TableRegistered.id);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    TableRegistered message = data is TableRegistered
        ? data
        : new TableRegistered.fromPds(data.toPds());

    if ((database == null || database == message.database) &&
        (table == null || table == message.table)) {
      return [_callback(message)];
    } else {
      return [];
    }
  }
}

/// Request record from a table
class RetrieveData extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_messages.v0.RetrieveData';

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Input key data class
  final String inkeyClass;

  /// Input key value
  final String inkeyValue;

  /// Retrieval parameters
  final Map<String, String> parameters;

  /// Requested keys by their index
  final List<int> keys;

  /// Requested values by their index
  final List<int> values;

  /// Constructor
  RetrieveData(this.database, this.table, this.inkeyClass, this.inkeyValue,
      {this.parameters: const {}, this.keys: const [], this.values: const []})
      : super(id);

  /// Construct from passive data structure.
  factory RetrieveData.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    String inkeyClass;
    String inkeyValue;
    Map<String, String> parameters;
    List<int> keys;
    List<int> values;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read inkeyClass.
    if (pds.containsKey('inkeyClass') && pds['inkeyClass'] is String) {
      inkeyClass = pds['inkeyClass'];
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'inkeyClass'");
    }

    // Read inkeyValue.
    if (pds.containsKey('inkeyValue') && pds['inkeyValue'] is String) {
      inkeyValue = pds['inkeyValue'];
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'inkeyValue'");
    }

    // Read parameters.
    if (pds.containsKey('parameters') && pds['parameters'] is Map) {
      parameters = new Map<String, String>.from(pds['parameters']);
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'parameters'");
    }

    // Read keys.
    if (pds.containsKey('keys') && pds['keys'] is List) {
      keys = new List<int>.from(pds['keys']);
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'keys'");
    }

    // Read values.
    if (pds.containsKey('values') && pds['values'] is List) {
      values = new List<int>.from(pds['values']);
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'values'");
    }

    return new RetrieveData(database, table, inkeyClass, inkeyValue,
        parameters: parameters, keys: keys, values: values);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'inkeyClass': inkeyClass,
      'inkeyValue': inkeyValue,
      'parameters': parameters,
      'keys': keys,
      'values': values
    };
  }
}

/// Message handler for RetrieveData
class RetrieveDataHandler extends OmnibusRequestHandler {
  /// Callback
  final OmnibusRequestCallback<RetrieveData> _callback;

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Constructor
  RetrieveDataHandler(this._callback, {this.database: null, this.table: null})
      : super(RetrieveData.id);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data) async {
    RetrieveData message =
        data is RetrieveData ? data : new RetrieveData.fromPds(data.toPds());

    if ((database == null || database == message.database) &&
        (table == null || table == message.table)) {
      return [_callback(message)];
    } else {
      return [];
    }
  }
}

/// Return data to the DataSever
///
/// This message contain the data that is retrieved in response to an
/// mRetrieveData request.
class DataRetrieved extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_messages.v0.DataRetrieved';

  /// Retrieved data records
  final List<DataRecord> records;

  /// Constructor
  DataRetrieved(this.records) : super(id);

  /// Construct from passive data structure.
  factory DataRetrieved.fromPds(final Map<String, dynamic> pds) {
    List<DataRecord> records;

    // Read records.
    if (pds.containsKey('records') && pds['records'] is List) {
      records = new List<DataRecord>.generate(
          pds['records'].length,
          (int i1) => new DataRecord.fromPds(
              new Map<String, dynamic>.from(pds['records'][i1])));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'records'");
    }

    return new DataRetrieved(records);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'records': toPdsList(records)};
  }
}

/// Create DataRetrieved Stream from the given responses.
Stream<DataRetrieved> streamDataRetrieved(OmnibusResponses responses) {
  return responses.stream(
      DataRetrieved.id,
      (OmnibusResponse data) => data is DataRetrieved
          ? data
          : new DataRetrieved.fromPds(data.toPds())) as Stream<DataRetrieved>;
}

/// Send data to the DataServer
class SubmitData extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.SubmitData';

  /// Records that are sumbitted
  final List<DataRecord> records;

  /// Constructor
  SubmitData(this.records) : super(id);

  /// Construct from passive data structure.
  factory SubmitData.fromPds(final Map<String, dynamic> pds) {
    List<DataRecord> records;

    // Read records.
    if (pds.containsKey('records') && pds['records'] is List) {
      records = new List<DataRecord>.generate(
          pds['records'].length,
          (int i1) => new DataRecord.fromPds(
              new Map<String, dynamic>.from(pds['records'][i1])));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'records'");
    }

    return new SubmitData(records);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'records': toPdsList(records)};
  }
}

/// Message handler for SubmitData
class SubmitDataHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<SubmitData> _callback;

  /// Constructor
  SubmitDataHandler(this._callback) : super(SubmitData.id);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    SubmitData message =
        data is SubmitData ? data : new SubmitData.fromPds(data.toPds());

    return [_callback(message)];
  }
}

/// Find data snippets
class FindSnippets extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_messages.v0.FindSnippets';

  /// Requested data snippets
  final List<SnippetSelector> snippets;

  /// Alternative starting key data class
  final String inkeyClass;

  /// Alternative starting key value
  final String inkeyValue;

  /// Constructor
  FindSnippets(this.snippets, {this.inkeyClass: '', this.inkeyValue: ''})
      : super(id);

  /// Construct from passive data structure.
  factory FindSnippets.fromPds(final Map<String, dynamic> pds) {
    List<SnippetSelector> snippets;
    String inkeyClass;
    String inkeyValue;

    // Read snippets.
    if (pds.containsKey('snippets') && pds['snippets'] is List) {
      snippets = new List<SnippetSelector>.generate(
          pds['snippets'].length,
          (int i1) => new SnippetSelector.fromPds(
              new Map<String, dynamic>.from(pds['snippets'][i1])));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'snippets'");
    }

    // Read inkeyClass.
    if (pds.containsKey('inkeyClass') && pds['inkeyClass'] is String) {
      inkeyClass = pds['inkeyClass'];
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'inkeyClass'");
    }

    // Read inkeyValue.
    if (pds.containsKey('inkeyValue') && pds['inkeyValue'] is String) {
      inkeyValue = pds['inkeyValue'];
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'inkeyValue'");
    }

    return new FindSnippets(snippets,
        inkeyClass: inkeyClass, inkeyValue: inkeyValue);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'snippets': toPdsList(snippets),
      'inkeyClass': inkeyClass,
      'inkeyValue': inkeyValue
    };
  }
}

/// Message handler for FindSnippets
class FindSnippetsHandler extends OmnibusRequestHandler {
  /// Callback
  final OmnibusRequestCallback<FindSnippets> _callback;

  /// Constructor
  FindSnippetsHandler(this._callback) : super(FindSnippets.id);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data) async {
    FindSnippets message =
        data is FindSnippets ? data : new FindSnippets.fromPds(data.toPds());

    return [_callback(message)];
  }
}

/// Return snippets to the data consumer.
class SnippetsFound extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_messages.v0.SnippetsFound';

  /// Resulting snippets
  final List<SnippetData> snippets;

  /// Constructor
  SnippetsFound(this.snippets) : super(id);

  /// Construct from passive data structure.
  factory SnippetsFound.fromPds(final Map<String, dynamic> pds) {
    List<SnippetData> snippets;

    // Read snippets.
    if (pds.containsKey('snippets') && pds['snippets'] is List) {
      snippets = new List<SnippetData>.generate(
          pds['snippets'].length,
          (int i1) => new SnippetData.fromPds(
              new Map<String, dynamic>.from(pds['snippets'][i1])));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'snippets'");
    }

    return new SnippetsFound(snippets);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'snippets': toPdsList(snippets)};
  }
}

/// Create SnippetsFound Stream from the given responses.
Stream<SnippetsFound> streamSnippetsFound(OmnibusResponses responses) {
  return responses.stream(
      SnippetsFound.id,
      (OmnibusResponse data) => data is SnippetsFound
          ? data
          : new SnippetsFound.fromPds(data.toPds())) as Stream<SnippetsFound>;
}

/// Search table using a text
class TextSearch extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_messages.v0.TextSearch';

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Search text
  final String text;

  /// Offset for paged search
  final int offset;

  /// Constructor
  TextSearch(this.database, this.table, this.text, this.offset) : super(id);

  /// Construct from passive data structure.
  factory TextSearch.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    String text;
    int offset;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read text.
    if (pds.containsKey('text') && pds['text'] is String) {
      text = pds['text'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'text'");
    }

    // Read offset.
    if (pds.containsKey('offset') && pds['offset'] is int) {
      offset = pds['offset'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'offset'");
    }

    return new TextSearch(database, table, text, offset);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'text': text,
      'offset': offset
    };
  }
}

/// Message handler for TextSearch
class TextSearchHandler extends OmnibusRequestHandler {
  /// Callback
  final OmnibusRequestCallback<TextSearch> _callback;

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Constructor
  TextSearchHandler(this._callback, {this.database: null, this.table: null})
      : super(TextSearch.id);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data) async {
    TextSearch message =
        data is TextSearch ? data : new TextSearch.fromPds(data.toPds());

    if ((database == null || database == message.database) &&
        (table == null || table == message.table)) {
      return [_callback(message)];
    } else {
      return [];
    }
  }
}

/// Return results of a text search.
class TextSearchResults extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_messages.v0.TextSearchResults';

  /// N best matches for the given search text
  final List<TextSearchMatch> matches;

  /// Constructor
  TextSearchResults(this.matches) : super(id);

  /// Construct from passive data structure.
  factory TextSearchResults.fromPds(final Map<String, dynamic> pds) {
    List<TextSearchMatch> matches;

    // Read matches.
    if (pds.containsKey('matches') && pds['matches'] is List) {
      matches = new List<TextSearchMatch>.generate(
          pds['matches'].length,
          (int i1) => new TextSearchMatch.fromPds(
              new Map<String, dynamic>.from(pds['matches'][i1])));
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'matches'");
    }

    return new TextSearchResults(matches);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'matches': toPdsList(matches)};
  }
}

/// Create TextSearchResults Stream from the given responses.
Stream<TextSearchResults> streamTextSearchResults(OmnibusResponses responses) {
  return responses.stream(
          TextSearchResults.id,
          (OmnibusResponse data) => data is TextSearchResults
              ? data
              : new TextSearchResults.fromPds(data.toPds()))
      as Stream<TextSearchResults>;
}

/// Try to switch to a new active record.
class PushActiveRecord extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.PushActiveRecord';

  /// Target key data class
  final String key;

  /// Target key value (encoded as String)
  final String value;

  /// Constructor
  PushActiveRecord(this.key, this.value) : super(id);

  /// Construct from passive data structure.
  factory PushActiveRecord.fromPds(final Map<String, dynamic> pds) {
    String key;
    String value;

    // Read key.
    if (pds.containsKey('key') && pds['key'] is String) {
      key = pds['key'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'key'");
    }

    // Read value.
    if (pds.containsKey('value') && pds['value'] is String) {
      value = pds['value'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'value'");
    }

    return new PushActiveRecord(key, value);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'key': key, 'value': value};
  }
}

/// Message handler for PushActiveRecord
class PushActiveRecordHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<PushActiveRecord> _callback;

  /// Constructor
  PushActiveRecordHandler(this._callback) : super(PushActiveRecord.id);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    PushActiveRecord message = data is PushActiveRecord
        ? data
        : new PushActiveRecord.fromPds(data.toPds());

    return [_callback(message)];
  }
}

/// Switch to a new active record.
class ActiveRecordChanged extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.ActiveRecordChanged';

  /// Target key data class
  final String key;

  /// Target key value (encoded as String)
  final String value;

  /// Constructor
  ActiveRecordChanged(this.key, this.value) : super(id);

  /// Construct from passive data structure.
  factory ActiveRecordChanged.fromPds(final Map<String, dynamic> pds) {
    String key;
    String value;

    // Read key.
    if (pds.containsKey('key') && pds['key'] is String) {
      key = pds['key'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'key'");
    }

    // Read value.
    if (pds.containsKey('value') && pds['value'] is String) {
      value = pds['value'];
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'value'");
    }

    return new ActiveRecordChanged(key, value);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'key': key, 'value': value};
  }
}

/// Message handler for ActiveRecordChanged
class ActiveRecordChangedHandler extends OmnibusEventHandler {
  /// Callback
  final OmnibusEventCallback<ActiveRecordChanged> _callback;

  /// Constructor
  ActiveRecordChangedHandler(this._callback) : super(ActiveRecordChanged.id);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data) async {
    ActiveRecordChanged message = data is ActiveRecordChanged
        ? data
        : new ActiveRecordChanged.fromPds(data.toPds());

    return [_callback(message)];
  }
}
